<?php
namespace GopNext\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class CalculationBudgetService {
    
    public function __construct($entityManager, $environment){
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
  
    /**
    * Start transaction on database server
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
  
    /**
    * Commit transaction on database server
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function commit() {
        $this->connection->commit();
    }
  
    /**
    * Roll back transaction in database
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function rollback() {
        $this->connection->rollback();
    }

    /**
    * Make changes to the monthly budget
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function changeBudgetMonth($params) {
        $paramsMonth = [
            "NRORCAMENTOGOP" => $params["NRORCAMENTOGOP"],
            "IDTIPOORCAOP"   => $params["IDTIPOORCAOP"],
            "IDEDITMES"      => $params["IDEDITMES"],
            "NRORG"          => $params["NRORG"]
        ];
                
        $query = 
            "SELECT I.IDTIPOORCAOP, I.NRITEMORCAOP, M.NRITORCAMESGOP, M.DTORCAMENTO,
                    M.IDALTMANUAL, TO_NUMBER(TO_CHAR(M.DTORCAMENTO,'MM')) NRMES
               FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                           AND I.IDTIPOORCAOP = :IDTIPOORCAOP
                                   INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                                                          AND TO_CHAR(M.DTORCAMENTO,'YYYY/MM') >= :IDEDITMES
              WHERE O.NRORCAMENTOGOP = :NRORCAMENTOGOP
                AND O.NRORG = :NRORG
              ORDER BY I.NRITEMORCAOP";
        try {
            $stmt = $this->connection->prepare($query);
            $stmt->execute($paramsMonth);
            $records = $stmt->fetchAll();
            $amount = 0;
            $nrItemOrcaOp = 0;
            
            foreach($records as $record){
                if ($nrItemOrcaOp !== 0 && $nrItemOrcaOp !== $record["NRITEMORCAOP"]) {
                    $this->updatesTotals($amount, $record["NRITEMORCAOP"], $params["NRORG"]);
                    $amount = 0;
                }
                $nrItemOrcaOp = $record["NRITEMORCAOP"];
                
                $update = "UPDATE ITORCAMESGOP SET VRORCAMENTO = :VRORCAMENTO
                            WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                              AND NRORG = :NRORG";
                
                $paramsUpd = [
                    "NRITORCAMESGOP" => $record["NRITORCAMESGOP"],
                    "NRORG" => $paramsMonth["NRORG"]
                ];
                switch ($record["NRMES"]) {
                    case 1:
                        $paramsUpd["VRORCAMENTO"] = $params["JANEIRO"];
                        break;
                    case 2:
                        $paramsUpd["VRORCAMENTO"] = $params["FEVEREIRO"];
                        break;
                    case 3:
                        $paramsUpd["VRORCAMENTO"] = $params["MARCO"];
                        break;
                    case 4:
                        $paramsUpd["VRORCAMENTO"] = $params["ABRIL"];
                        break;
                    case 5:
                        $paramsUpd["VRORCAMENTO"] = $params["MAIO"];
                        break;
                    case 6:
                        $paramsUpd["VRORCAMENTO"] = $params["JUNHO"];
                        break;
                    case 7:
                        $paramsUpd["VRORCAMENTO"] = $params["JULHO"];
                        break;
                    case 8:
                        $paramsUpd["VRORCAMENTO"] = $params["AGOSTO"];
                        break;
                    case 9:
                        $paramsUpd["VRORCAMENTO"] = $params["SETEMBRO"];
                        break;
                    case 10:
                        $paramsUpd["VRORCAMENTO"] = $params["OUTUBRO"];
                        break;
                    case 11:
                        $paramsUpd["VRORCAMENTO"] = $params["NOVEMBRO"];
                        break;
                    case 12:
                        $paramsUpd["VRORCAMENTO"] = $params["DEZEMBRO"];
                        break;
                }
                $amount = $amount + $paramsUpd["VRORCAMENTO"];
                
                $stmt = $this->connection->prepare($update);
                $stmt->execute($paramsUpd);
            }
            
            $this->updatesTotals($amount, $nrItemOrcaOp, $params["NRORG"]);
        } catch (Exception $ex) {
            throw $ex;
        };
    }

    /**
    * Updates totals regarding changes applied in the month
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function updatesTotals($value, $nrItemOrcaOp, $nrOrg) {
        try {
            
            $update = "UPDATE ITORCAMESGOP SET VRTOTALMES = :VRTOTALMES
                        WHERE NRITEMORCAOP = :NRITEMORCAOP
                          AND NRORG = :NRORG";
                
            $paramsUpd = [
                "VRTOTALMES"   => $value,
                "NRITEMORCAOP" => $nrItemOrcaOp,
                "NRORG"        => $nrOrg
            ];
            
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
            
            $update = "UPDATE ITEMORCAGOP SET VRTOTALMES = :VRTOTALMES
                        WHERE NRITEMORCAOP = :NRITEMORCAOP
                          AND NRORG = :NRORG";
                
            $paramsUpd = [
                "VRTOTALMES"   => $value,
                "NRITEMORCAOP" => $nrItemOrcaOp,
                "NRORG"        => $nrOrg
            ];
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
        } catch (Exception $ex) {
            throw $ex;
        }
    }

    /**
    * Triggers budget calculations such as calculation by percentage, by previous periods,
    * among others used in the CALCULA_ORCAMENTO_GOP procedure
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function budgetCalculation($data, $nrLista) {
        try {
            $procedure = new StoredProcedure($this->connection, 'CALCULA_ORCAMENTO_GOP');
            
            $procedure->addParam(new Param('P_NRANOORCAGOP', Param::PARAM_INPUT, $data['NRANOORCAGOP'], Param::PARAM_TYPE_INT));
	        $procedure->addParam(new Param('P_IDPERIODOORCA', Param::PARAM_INPUT, $data['IDPERIODOORCA'], Param::PARAM_TYPE_STR));
	        $procedure->addParam(new Param('P_NRPERIODOORCA', Param::PARAM_INPUT, $data['NRPERIODOORCA'], Param::PARAM_TYPE_INT));
	        $procedure->addParam(new Param('P_IDTIPOORCAOP', Param::PARAM_INPUT, $data['IDTIPOORCAOP'], Param::PARAM_TYPE_STR));
	        
	        $procedure->addParam(new Param('P_PERCENTUAL', Param::PARAM_INPUT, $data['CALCULATION'] === "P" ? "S" : "N", Param::PARAM_TYPE_STR));
	        $procedure->addParam(new Param('P_LISTAFIL', Param::PARAM_INPUT, $nrLista, Param::PARAM_TYPE_STR));
	        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR)); 
	        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT,null,Param::PARAM_TYPE_STR));
	        $procedure->execute();
        } catch (Exception $ex) {
            throw $ex;
        }
    }
  
    public function completeBudget($data, $nrLista) {
        try {
            // Implementar
        } catch (Exception $ex) {
            throw $ex;
        }
    }

    /**
    * Update responsible for applying changes made to the budget details.
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function changeBudgetDetailMonth($params) {
        try {
            $update = "UPDATE ITDETALORCAMESGOP
                          SET VRORCAMENTO = :VRORCAMENTO,
                              QTORCAMENTO = :QTORCAMENTO
                        WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        };
    }

    /**
    * Update responsible for updating the monthly totalizers.
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function updateTotalMonth($params) {
        try {
            $update = "UPDATE ITORCAMESGOP
                          SET VRORCAMENTO = (SELECT SUM(VRORCAMENTO)
                                               FROM ITDETALORCAMESGOP
                                              WHERE NRITORCAMESGOP = :NRITORCAMESGOP)
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        };
    }
    
    /**
    * 
    * 
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function cleanDetailing($data) {
        try {
            $paramsUpd = [
                "NRITORCAMESGOP" => $data["NRITORCAMESGOP"],
                "NRORG" => $data[0]["value"]
            ];
                
            $update = "UPDATE ITDETALORCAMESGOP
                          SET VRORCAMENTO = 0,
                              QTORCAMENTO = 0
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
                
            $update = "UPDATE ITORCAMESGOP SET VRORCAMENTO = 0
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
        } catch (Exception $ex) {
            throw $ex;
        }
    }

    /**
    * Update responsible for applying the change to the total value of the detail option.
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function changeBudgetDetMonthTotal($params) {
        try {
            $update = "UPDATE ITORCAMESGOP
                          SET VRORCAMENTO = :VRORCAMENTO
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        };
    }
}