import {
  IEventParam,
  Metadata,
  Http,
  Loader,
} from '@zeedhi/core';
import {
  TekGrid,
} from '@zeedhi/teknisa-components-common';
import {
  Button,
  GridEditable,
  LoadingService,
  Text,
  Select,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class PercentageAndStructureForBudgetController {
  // private departmentCurrentRow: any = {};
  
  private employeeCurrentRow: any = {};
  
  public isNotUpdateRowEditing = true;
  
  // public isNotInlineSelectsWithRequestEditing = true;
  
  public previewStructure = 1;
  
  public rowsEditable = [];

  public onInlineEditUpdateRow({ component }: IEventParam<GridEditable>) {
    this.isNotUpdateRowEditing = !component.editing;
  }
  
  public createModalDetail() {
    const detailsModal = Loader.getInstance('DetailsItemOperationalModalController');
    detailsModal.createDetailsItemOperationalModal();
  }

  public async cancelEditingUpdateRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.onInlineEditUpdateRow({ component: grid });
    grid.editingNewRows = false;
  }

  public async saveEditingUpdateRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.onInlineEditUpdateRow({ component: grid });
    grid.editingNewRows = false;
  }

  /* get isInlineSelectsWithRequestEditing() {
    return !this.isNotInlineSelectsWithRequestEditing;
  } */

  /* public onInlineEditSelectsWithRequest({ component }: IEventParam<GridEditable>) {
    this.isNotInlineSelectsWithRequestEditing = !component.editing;
  } */

  /* public async cancelEditingSelectsWithRequest({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.onInlineEditSelectsWithRequest({ component: grid });
  } */

  /* public async saveEditingSelectsWithRequest({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.onInlineEditSelectsWithRequest({ component: grid });
  } */
  
  /* public addNewRow({ component }: any) {
    const gridDetailItens = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const grid: GridEditable = component.parent;
    const newRow = {
    };
    console.warn(gridDetailItens);
    grid.columns[2].editable = true;
    grid.editingNewRows = true;
    grid.addNewRow(newRow);
  } */
  
  public structureItemChange({ component }: IEventParam<Button>) {
    const utilities = new Utilities();
    const grid = component.parent as TekGrid;
    const data = grid.getEditedRows();
    LoadingService.show();
    Http.post('/structureItemChange', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      await grid.saveEditedRows();
      this.onInlineEditUpdateRow({ component: grid });
      LoadingService.hide();
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public unitRowClick({ component }: any) {
    const selectedUnit = `${component.datasource.currentRow.CDFILIAL} - ${component.datasource.currentRow.NMFILIAL}`;
    const routeName = 'operationalItemStructure';
    const grid = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const unit = Metadata.getInstance<Text>('selectedUnit');
    
    const sendObject = {
      filter: [{
        name: 'CDFILIAL',
        operator: '=',
        value: component.datasource.currentRow.CDFILIAL,
      }],
    };
    let data;
    
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      data = response.data.dataset[routeName];
      
      /*
      if (data.some((obj) => ('IDTIPO' in obj))) {
        data.forEach((obj) => {
          obj.IDTIPO = I18n.translate('gop::T_'.concat(obj.IDTIPO));
        });
      }
      */
      grid.datasource.setFilter([]);
      
      grid.datasource.updateData(data);
      
      LoadingService.hide();
    }).catch((e) => {
      // utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
      LoadingService.hide();
      console.warn(e);
    });
    
    unit.text = selectedUnit;
  }
  
  public rowOperationalItemClick({ component }: any) {
    const routeName = 'detailsOperationalItemsStructure';
    const grid = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const { currentRow } = component.datasource;
    if (currentRow.NRSEQ === 11) {
      component.parent.height = '96%';
      grid.parent.isVisible = false;
    } else {
      const sendObject = {
        filter: [{
          name: 'NRSEQITEMOP',
          operator: '=',
          value: currentRow.NRSEQ,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: currentRow.CDFILIAL,
        }],
      };
      LoadingService.show();
      Http.post(`/${routeName}`, sendObject).then((response) => {
        const data = response.data.dataset[routeName];
        grid.datasource.setFilter([]);
        grid.datasource.updateData(data);
        this.setSourceOptions(currentRow.NRSEQITEMORCA);
        LoadingService.hide();
      }).catch((e) => {
        LoadingService.hide();
        console.warn(e);
      });
      
      component.parent.height = '30%';
      grid.parent.isVisible = true;
    }
  }
  
  public setSourceOptions(nrSeqItemOrca) {
    const grid = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const serviceOption = { name: 'Serviço', value: 'SERVICO' };
    const productOption = { name: 'Produto', value: 'PRODUTO' };
    const subProductOption = { name: 'Subgrupo', value: 'SUBGRUPOPROD' };
    const billToPayOption = { name: 'Conta a Pagar', value: 'TCPAGAR' };
    const accountOption = { name: 'Conta Contábil', value: 'CONTABIL' };
    const tipoOpOption = { name: 'Tipo Operação', value: 'TIPOOPERAC' };
    const origin = [
      { NRSEQITEM: [0], DATA: [{ name: 'Diversos', value: 'DIVERSOS' }] },
      { NRSEQITEM: [1, 8, 9], DATA: [{ name: 'Imposto', value: 'IMPOSTO' }] },
      { NRSEQITEM: [2], DATA: [] },
      { NRSEQITEM: [3], DATA: [{ name: 'Mão de Obra', value: 'ORCAMENTOMO' }] },
      { NRSEQITEM: [5, 6, 7], DATA: [{ name: 'Despesas', value: 'DESPESA' }] },
      { NRSEQITEM: [10], DATA: [{ name: 'Investimentos', value: 'INVESTIMENTOS' }] },
      { NRSEQITEM: [11], DATA: [{ name: 'Depreciação', value: 'DEPRECIACAO' }] },
    ];
    const originItem = origin.filter((item) => (item.NRSEQITEM.indexOf(nrSeqItemOrca) !== -1));
    const data = originItem[0].DATA;
    switch (nrSeqItemOrca) {
      case 0:
      case 2:
        data.push(serviceOption);
        data.push(productOption);
        data.push(subProductOption);
        break;
      case 3:
        data.push(billToPayOption);
        data.push(accountOption);
        break;
      case 6:
        data.push(productOption);
        data.push(subProductOption);
        data.push(billToPayOption);
        data.push(accountOption);
        break;
      case 7:
        data.push(tipoOpOption);
        break;
      default:
        console.warn();
    }
    const columns = grid.columns;
    const columnOrigin = columns.filter((item) => (item.name === 'DSORIGEM'));
    columnOrigin[0].componentProps.datasource.data = data;
  }
  
  public setRouteDescription({ row, component }) {
    const gridUnits = Metadata.getInstance<TekGrid>('gridUnits');
    const gridItems = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const originRow = component ? component.value : row.DSORIGEM;
    const cdFilial = gridUnits.datasource.currentRow.CDFILIAL;
    const nrSeqItem = gridItems.datasource.currentRow.NRSEQITEMORCA;
    const routeDescription = [
      // FATURAMENTO (FATURAMENTO DIVERSO)----- OK ------
      {
        NRSEQITEM: [0], ORIGIN: 'DIVERSOS', route: '/crudN/descriptionBilling', params: {},
      },
      // IMPOSTO , IMPOSTO A RECUPERAR, IMPOSTO SOBRE O LUCRO ------- OK ------
      {
        NRSEQITEM: [1, 8, 9], ORIGIN: 'IMPOSTO', route: '/crudN/descriptionTax', params: {},
      },
      // MAO DE OBRA ----- OK -----------
      {
        NRSEQITEM: [3], ORIGIN: 'ORCAMENTOMO', route: '/crudN/descriptionLabor', params: { CDFILIAL: cdFilial },
      },
      // DESPESAS DA MATRIZ ------ OK ------
      {
        NRSEQITEM: [5], ORIGIN: 'DESPESA', route: '/crudN/descriptionHeadOfficeExpenses', params: {},
      },
      // DESPESAS DIVERSAS ------ OK -------
      {
        NRSEQITEM: [6], ORIGIN: 'DESPESA', route: '/crudN/descriptionMiscellaneousExpenses', params: {},
      },
      // DESPESAS LANCAMENTO ENTRADA DO TIPO DESPESA ------- OK -------
      {
        NRSEQITEM: [7], ORIGIN: 'DESPESAS', route: '/crudN/descriptionExpensesEntryReleaseDesp', params: {},
      },
      // DESPESAS LANCAMENTO ENTRADA DO TIPO OPERACAO ------ OK -----
      {
        NRSEQITEM: [7], ORIGIN: 'TIPOOPERAC', route: '/crudN/descriptionExpensesEntryReleaseOp', params: {},
      },
      // INVESTIMENTOS ----- OK -------
      {
        NRSEQITEM: [10], ORIGIN: 'INVESTIMENTOS', route: '/crudN/descriptionInvestment', params: {},
      },
      // DEPRECIACAO
      {
        NRSEQITEM: [11], ORIGIN: 'DEPRECIACAO', route: '/crudN/descriptionDepreciation', params: {},
      },
      // SERVICO - FATURAMENTO OU CONSUMO ------- OK ------
      {
        NRSEQITEM: [0, 2], ORIGIN: 'SERVICO', route: '/crudN/descriptionService', params: { CDFILIAL: cdFilial },
      },
      // PRODUTO - FATURAMENTO, CONSUMO, DESPESAS DIVERSAS ----- OK -----
      {
        NRSEQITEM: [0, 2, 6], ORIGIN: 'PRODUTO', route: '/crudN/descriptionProduct', params: {},
      },
      // SUBGRUPO DE PRODUTO - FATURAMENTO, CONSUMO, DESPESA DIVERSA ----
      {
        NRSEQITEM: [0, 2, 6], ORIGIN: 'SUBGRUPOPROD', route: '/crudN/descriptionSubGroup', params: {},
      },
      // CONTA CONTABIL - DESPESAS DIVERSAS , LANC ENTRA, MAO OBRA ---- OK ----
      {
        NRSEQITEM: [3, 6, 7], ORIGIN: 'CONTABIL', route: '/crudN/descriptionAccount', params: {},
      },
      // CONTA A PAGAR - DESPESAS DIVERSAS , LANC ENTRA, MAO OBRA ---- OK ----
      {
        NRSEQITEM: [3, 6, 7], ORIGIN: 'TCPAGAR', route: '/crudN/descriptionBillsToPay', params: {},
      },
    ];
    const result = routeDescription.filter((item) => item.NRSEQITEM.indexOf(nrSeqItem) !== -1 && item.ORIGIN === originRow);
    const route = result.length > 0 ? result[0].route : '';
    const params = result.length > 0 ? result[0].params : {};
    if (component) {
      const gridDetail = Metadata.getInstance<TekGrid>('detailsOperationalItems');
      const fieldDescription = gridDetail.getComponent(row.NRSEQUENCIAL, 'DSDESCRICAO');
      (fieldDescription as any).datasource.route = route;
      (fieldDescription as any).datasource.filter = params;
    }
    return {
      route,
      uniqueKey: 'CDCODIGO',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: params,
    };
  }
  
  /* public onChangeEmployeeCurrentRow({ component }: IEventParam<Datasource>) {
    this.employeeCurrentRow = component.currentRow;
  } */
  
  public onChangePreviewStructure({ component }) {
    if (component.datasource.currentRowValue) {
      this.previewStructure = component.datasource.currentRowValue.value;
    }
  }
  
  public setPreviewStructure({ component }) {
    component.setValue(this.previewStructure);
  }
  
  get unitView() {
    return this.previewStructure === 1;
  }
  
  get detailedView() {
    return this.previewStructure === 2;
  }
  
  public cellIsEditableItem({ row }) {
    return row.NRSEQ !== 0;
  }
  
  public budgetUnits({ component }: any) {
    const routeName = 'budgetUnits';
    const sendObject = {};
    
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      component.datasource.setFilter([]);
      component.datasource.updateData(response.data.dataset[routeName]);
        
      LoadingService.hide();
    }).catch((e) => {
      // utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public addNewRow({ component }: IEventParam<TekGrid>) {
    const newId = (component.datasource.total + 1).toString();
    this.rowsEditable.push(newId);
    component.addNewRow({ NRSEQUENCIAL: newId }, 'start').then(() => {
      const nameInput = Metadata.getInstance<Select>(`DSORIGEM_editable_${newId}`);
      nameInput.setFocus();
    });
  }
  
  public editableOrigin({ row }) {
    const newRow = this.rowsEditable.indexOf(row.NRSEQUENCIAL) !== -1;
    return newRow;
  }
}
