import {
  Metadata,
  IEventParam,
  I18n,
  Http,
} from '@zeedhi/core';
import {
  Modal,
  ModalService,
  SelectMultiple,
  Button,
  LoadingService,
} from '@zeedhi/common';
import BudgetParameters from '@/controllers/BudgetParameters';
import { ZdTabs, ZdForm, ZdCol } from '@zeedhi/vuetify';
import { Utilities } from '@/util/Utilities';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class BreakdownOfDetailBudgetModalController {
  private breakdownOfDetailBudgetModal!: Modal;
  
  public clean = false;
  
  public isEditing = true;
  
  private dataDiners = [];
  
  public title: string = ' ';
  
  public async createBreakdownOfDetailBudgetModal() {
    this.breakdownOfDetailBudgetModal = await ModalService.createFromJson('/modals/breakdownOfDetailBudgetModal', true);
    this.breakdownOfDetailBudgetModal.show();
  }
  
  public destroyBreakdownOfDetailBudgetModal() {
    this.breakdownOfDetailBudgetModal?.destroy();
  }
  
  public async openBreakdownOfDetailBudgetModal({ row }: IEventParam<Button>) {
    if (!this.breakdownOfDetailBudgetModal) {
      await this.createBreakdownOfDetailBudgetModal();
    } else {
      await this.breakdownOfDetailBudgetModal.show();
    }
    this.serviceDetails(row.NRITDETALORCAMESGOP);
  }
  
  public closeBreakdownOfDetailBudgetModal() {
    this.breakdownOfDetailBudgetModal.hide();
  }
  
  public defineTitle() {
    this.title = 'Unidade'.concat(':', '0001 - Teste | Faturamento | Janeiro');
  }
  
  /**
  * Triggers the function that defines the pages as not loaded, and leaves the first visible page active.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public clearData() {
    const tabsBudget = Metadata.getInstance<ZdTabs>('tabsBudget');
    BudgetParameters.setNotRecharged();
    tabsBudget.activeTab = tabsBudget.tabs.findIndex((obj) => (obj.isVisible === true));
  }
  
  /**
  * Updates the search for units in the filter according to the fields entered.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public createDataSourceFilter() {
    const filial = Metadata.getInstance<SelectMultiple>('CDFILIAL');
    const grupFili = Metadata.getInstance<SelectMultiple>('CDGRUPFILI');
    const areaGere = Metadata.getInstance<SelectMultiple>('CDAREAGERE');
    const segMerc = Metadata.getInstance<SelectMultiple>('CDSEGMER');
    
    filial.datasource.setFilter({
      CDGRUPFILI: grupFili.value && grupFili.value.length ? grupFili.value.join('_') : 'T',
      CDAREAGERE: areaGere.value && areaGere.value.length ? areaGere.value.join('_') : 'T',
      CDSEGMER: segMerc.value && segMerc.value.length ? segMerc.value.join('_') : 'T',
    });
  }
  
  public async serviceDetails(nrItDdetalOrcaMesGOP) {
    const routeName = 'gop10000_serviceDetails';
    const utilities = new Utilities();
    const sendObject = {
      filter: [{
        name: 'NRITDETALORCAMESGOP',
        operator: '=',
        value: nrItDdetalOrcaMesGOP,
      }],
    };
    const gridDiet = Metadata.getInstance<TekGrid>('gridDiet');
    const colDiet = Metadata.getInstance<ZdCol>('colDiet');
    const colOperationalItem = Metadata.getInstance<ZdCol>('colOperationalItem');
    
    LoadingService.show();
    try {
      const response = await Http.post(`/${routeName}`, sendObject);
      const data = response.data.dataset[routeName];
      const dataDiet = response.data.dataset[routeName.concat('Diet')];
        
      this.dataDiners = data;
      
      if (data && data[0] && dataDiet && data[0].IDTIPOSERV === 'D') {
        const curDiet = data.filter((diet) => (diet.CDDIETPAD === dataDiet[0].CDDIETPAD));
        
        await gridDiet.cancelEditedRows();
        await gridDiet.datasource.updateData([]);
        await gridDiet.datasource.updateData(dataDiet);
        
        colDiet.cols = 3;
        colOperationalItem.cols = 9;
        
        this.updateForm(curDiet[0]);
      } else {
        await gridDiet.cancelEditedRows();
        await gridDiet.datasource.updateData([]);
        
        colDiet.cols = 0;
        colOperationalItem.cols = 12;
      }
      
      LoadingService.hide();
    } catch (e) {
      utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
      LoadingService.hide();
      console.warn(e);
    }
  }
  
  public updateForm(data) {
    const formDiners = Metadata.getInstance<ZdForm>('gridFormDetServ');
    
    formDiners.value.VRCUSTPADR = data.VRCUSTPADR;
    
    formDiners.value.QTSEG = data.QTSEG;
    formDiners.value.TOTSEG = data.VRTOTSEG;
    
    formDiners.value.QTTER = data.QTTER;
    formDiners.value.TOTTER = data.VRTOTTER;
    
    formDiners.value.QTQUA = data.QTQUA;
    formDiners.value.TOTQUA = data.VRTOTQUA;
    
    formDiners.value.QTQUI = data.QTQUI;
    formDiners.value.TOTQUI = data.VRTOTQUI;
    
    formDiners.value.QTSEX = data.QTSEX;
    formDiners.value.TOTSEX = data.VRTOTSEX;
    
    formDiners.value.QTSAB = data.QTSAB;
    formDiners.value.TOTSAB = data.VRTOTSAB;
    
    formDiners.value.QTDOM = data.QTDOM;
    formDiners.value.TOTDOM = data.VRTOTDOM;
  }
  
  public dietRowClick({ row }: any) {
    const curDiet = this.dataDiners.filter((diet) => (diet.CDDIETPAD === row.CDDIETPAD));
    
    this.updateForm(curDiet[0]);
  }
  
  public cancelChanges() {
    
  }
}
